# -*- coding: utf-8 -*-
import itertools

from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.utils.text import slugify
from django.db import models

from imagekit.models import ProcessedImageField

from pages.utils import constants
from pages.utils.image import get_slider_sizes, upload_picture_to
from pages.utils.database import SEOMixin, AuditableMixin, ActivatedMixin, ActivatedQuerySet


class CustomQuerySet(ActivatedQuerySet):
    def get_queryset(self):
        return ActivatedQuerySet(self.model, using=self._db)


class Page(AuditableMixin, ActivatedMixin):
    FOUR = 4
    SIX = 6

    LOT = (
        (FOUR, '4'),
        (SIX, '6')
    )

    PROVIDERS = (
        ('Wistia', 'Wistia'),
        ('Vimeo', 'Vimeo'),
    )

    slug = models.SlugField(max_length=230, null=True)
    description = models.CharField(max_length=100, null=True)
    title = models.CharField(max_length=200, null=True, blank=True)

    video_hash = models.CharField(max_length=200, null=True, blank=True)
    video_provider = models.CharField(
        max_length=200,
        null=True,
        blank=True,
        choices=PROVIDERS
    )

    title_custom_color = models.CharField(max_length=7, null=True, blank=True)
    subtitle = models.CharField(max_length=200, null=True, blank=True)
    subtitle_custom_color = models.CharField(max_length=7, null=True, blank=True)
    timer_id = models.SlugField(max_length=100, null=True, blank=True)
    social_buttons = models.BooleanField(default=True)
    page_size = models.SmallIntegerField(default=9)
    product_categories = models.CharField(max_length=300, null=True, blank=True)
    num_bundle_elements = models.IntegerField(
        'Quantity of elements for the bundle',
        choices=LOT,
        null=True,
        blank=True)

    external_url = models.URLField(
        verbose_name='External URL',
        max_length=500,
        blank=True,
    )

    page_type = models.SmallIntegerField(
        default=1,
        choices=constants.PAGE_TYPES,
        blank=True,
    )

    #Temp

    banner_pc = ProcessedImageField(
        upload_to=upload_picture_to,
        format=constants.PAGE_SLIDER_FORMAT,
        options=constants.PAGE_SLIDER_OPTIONS,
        null=False,
        blank=True,
    )

    banner_tablet = ProcessedImageField(
        upload_to=upload_picture_to,
        format=constants.PAGE_SLIDER_FORMAT,
        options=constants.PAGE_SLIDER_OPTIONS,
        null=True,
        blank=True,
    )

    banner_mobile = ProcessedImageField(
        upload_to=upload_picture_to,
        format=constants.PAGE_SLIDER_FORMAT,
        options=constants.PAGE_SLIDER_OPTIONS,
        null=True,
        blank=True,
    )

    objects = CustomQuerySet.as_manager()

    def __unicode__(self):
        return u'%s' % self.title

    class Meta:
        db_table = 'pages'


class PageSEO(AuditableMixin, ActivatedMixin):
    slug = models.SlugField()
    title = models.CharField(max_length=250)
    description = models.CharField(max_length=250)
    keywords = models.CharField(max_length=250)
    page_type = models.CharField(max_length=250)
    banner_share = ProcessedImageField(
        upload_to=upload_picture_to,
        format=constants.PAGE_SLIDER_FORMAT,
        options=constants.PAGE_SLIDER_OPTIONS,
        null=True,
        blank=True,
    )
    canonical_url = models.URLField()

    class Meta:
        db_table = 'pages_seo'


class PageTimer(AuditableMixin, ActivatedMixin):
    identifier = models.SlugField()
    title = models.CharField(max_length=250)
    expire_date = models.DateTimeField()

    class Meta:
        db_table = 'pages_timer'


class PageTag(SEOMixin, AuditableMixin, ActivatedMixin):
    page = models.ForeignKey('Page')

    title = models.CharField(
        max_length=200,
    )

    slug = models.SlugField(
        max_length=230,
    )


@receiver(pre_save, sender=Page)
def pre_save_page(sender, instance=None, **kwargs):
    if constants.PAGE_AUTOSLUG:
        instance.slug = orig = slugify(instance.title)

        for x in itertools.count(1):
            if not Page.objects.filter(slug=instance.slug).exists():
                break
            instance.slug = '%s-%d' % (orig, x)

    return instance


@receiver(pre_save, sender=PageTag)
def pre_save_page_tag(sender, instance=None, **kwargs):
    if constants.PAGE_AUTOSLUG:
        instance.slug = orig = slugify(instance.title)

        for x in itertools.count(1):
            if not Page.objects.filter(slug=instance.slug).exists():
                break
            instance.slug = '%s-%d' % (orig, x)

    return instance


from django.contrib import admin

from pages.models.page import Page, PageSEO, PageTag, PageTimer
from pages.utils.constants import PAGE_CUSTOM_ADMIN
from pages.models.slider import Slider, SliderContent, SliderItem, SliderContentType


class PageTimerAdmin(admin.ModelAdmin):
    model = PageTimer
    list_display = ('identifier', 'title', 'expire_date', 'created', 'modified')
    list_filter = ('is_active', )
    search_fields = ('identifier', 'title', 'expire_date')


class PageSEOAdmin(admin.ModelAdmin):
    model = PageSEO
    list_display = ('title', 'description', 'slug', 'keywords', 'banner_share')


class PageAdmin(admin.ModelAdmin):
    list_display = ['description', 'external_url', 'is_active', 'created']
    exclude = ['created', 'updated']
    search_fields = ('descrition', 'title', 'subtitle')


class SliderAdmin(admin.ModelAdmin):
    list_display = ['title', 'is_active', 'created']
    list_filter = ['page', 'is_active']
    exclude = ['created', 'updated']
    search_fields = ('title', )


class SliderItemAdmin(admin.ModelAdmin):
    list_display = ['slider', 'active_from', 'active_to', 'is_active', 'created']
    exclude = ['created', 'updated']
    list_filter = ['slider', 'is_active']


class SliderContentTypeAdmin(admin.ModelAdmin):
    list_display = ['name']


class SliderContentAdmin(admin.ModelAdmin):
    list_display = ['order', 'type_content']


if not PAGE_CUSTOM_ADMIN:
    admin.site.register(Page, PageAdmin)

admin.site.register(Slider, SliderAdmin)
admin.site.register(SliderItem, SliderItemAdmin)
admin.site.register(SliderContent, SliderContentAdmin)
admin.site.register(SliderContentType, SliderContentTypeAdmin)
admin.site.register(PageSEO, PageSEOAdmin)
admin.site.register(PageTimer, PageTimerAdmin)



# -*- coding: utf-8 -*-
from django.db import models
from imagekit.models import ProcessedImageField

from pages.utils.database import AuditableMixin, ActivatedMixin
from pages.utils.image import get_slider_sizes, upload_picture_to
from pages.managers import SliderManager
from pages.utils import constants


class Slider(AuditableMixin):

    page = models.ForeignKey('Page')

    title = models.CharField(
        max_length=200,
    )

    is_active = models.BooleanField(
        default=False
    )

    dots = models.BooleanField(
        default=False
    )

    speed = models.IntegerField(
        default=500
    )

    objects = SliderManager()

    def __str__(self):
        return "({}) {}".format(self.page.title, self.title)

    class Meta:
        db_table = 'sliders'


class SliderItem(AuditableMixin, ActivatedMixin):

    slider = models.ForeignKey('Slider')

    picture_pc = ProcessedImageField(
        upload_to=upload_picture_to,
        processors=get_slider_sizes(constants.PAGE_SLIDER_PC_SIZES),
        format=constants.PAGE_SLIDER_FORMAT,
        options=constants.PAGE_SLIDER_OPTIONS,
    )

    picture_tablet = ProcessedImageField(
        upload_to=upload_picture_to,
        processors=get_slider_sizes(constants.PAGE_SLIDER_TABLET_SIZES),
        format=constants.PAGE_SLIDER_FORMAT,
        options=constants.PAGE_SLIDER_OPTIONS,
        null=True
    )

    picture_mobile = ProcessedImageField(
        upload_to=upload_picture_to,
        processors=get_slider_sizes(constants.PAGE_SLIDER_MOBILE_SIZES),
        format=constants.PAGE_SLIDER_FORMAT,
        options=constants.PAGE_SLIDER_OPTIONS,
        null=True
    )

    active_from = models.DateTimeField()

    active_to = models.DateTimeField()

    order = models.SmallIntegerField(
        default=1
    )

    @property
    def absolute_url_picture_pc(self):
        url = '%s/%s' %(constants.SITE_URL, self.picture_pc)
        return url

    @property
    def absolute_url_picture_tablet(self):
        url = '%s/%s' % (constants.SITE_URL, self.picture_tablet)
        return url

    @property
    def absolute_url_picture_mobile(self):
        url = '%s/%s' % (constants.SITE_URL, self.picture_mobile)
        return url

    def __str__(self):
        return "({}) {}".format(self.slider.title, self.picture_pc)

    class Meta:
        unique_together = ('order', 'slider',)
        db_table = 'slider_items'


class SliderContentType(models.Model):

    name = models.CharField(
        max_length=100,
    )

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'slider_content_types'


class SliderContent(models.Model):

    slider_item = models.ForeignKey('SliderItem')

    type_content = models.ForeignKey('SliderContentType')

    content = models.TextField()

    config = models.TextField()

    order = models.SmallIntegerField(
        default=1
    )

    def __str__(self):
        return "({}) {}".format(self.order, self.type_content.name)

    class Meta:
        unique_together = ('order', 'slider_item',)
        db_table = 'slider_contents'




from django.db import models


class SliderManager(models.Manager):
    """
        Manager for Slider class
    """
    def activated(self):
        """
            return only activated sliders
        """
        return self.filter(is_active=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0016_page_num_bundle_elements'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='video_hash',
            field=models.CharField(max_length=200, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='page',
            name='video_provider',
            field=models.CharField(max_length=200, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='num_bundle_elements',
            field=models.IntegerField(blank=True, null=True, verbose_name=b'Quantity of elements for the bundle', choices=[(4, b'4'), (6, b'6')]),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0007_auto_20170615_1504'),
    ]

    operations = [
        migrations.RenameField(
            model_name='page',
            old_name='picture_mobile',
            new_name='banner_mobile',
        ),
        migrations.RenameField(
            model_name='page',
            old_name='picture_pc',
            new_name='banner_pc',
        ),
        migrations.RenameField(
            model_name='page',
            old_name='picture_share',
            new_name='banner_share',
        ),
        migrations.RenameField(
            model_name='page',
            old_name='picture_tablet',
            new_name='banner_tablet',
        ),
        migrations.RenameField(
            model_name='page',
            old_name='type',
            new_name='page_type',
        ),
        migrations.AddField(
            model_name='page',
            name='subtitle',
            field=models.CharField(max_length=200, null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='slug',
            field=models.SlugField(max_length=230, null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='pageseo',
            name='url',
            field=models.SlugField(),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='pagetimer',
            name='identifier',
            field=models.SlugField(),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import pages.utils.image
import imagekit.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0008_auto_20170616_0136'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='page',
            name='seo_desc',
        ),
        migrations.RemoveField(
            model_name='page',
            name='seo_keywords',
        ),
        migrations.RemoveField(
            model_name='page',
            name='seo_sub_title',
        ),
        migrations.RemoveField(
            model_name='page',
            name='seo_title',
        ),
        migrations.AlterField(
            model_name='page',
            name='banner_mobile',
            field=imagekit.models.fields.ProcessedImageField(null=True, upload_to=pages.utils.image.upload_picture_to, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='banner_pc',
            field=imagekit.models.fields.ProcessedImageField(upload_to=pages.utils.image.upload_picture_to, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='banner_share',
            field=imagekit.models.fields.ProcessedImageField(null=True, upload_to=pages.utils.image.upload_picture_to, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='banner_tablet',
            field=imagekit.models.fields.ProcessedImageField(null=True, upload_to=pages.utils.image.upload_picture_to, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='page_type',
            field=models.SmallIntegerField(default=1, blank=True, choices=[(1, b'General')]),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='subtitle',
            field=models.CharField(max_length=200, null=True, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils import timezone


class SEOMixin(models.Model):

    seo_title = models.CharField(
        max_length=100,
    )

    seo_sub_title = models.CharField(
        max_length=100,
    )

    seo_desc = models.CharField(
        max_length=100,
    )

    seo_keywords = models.CharField(
        max_length=100,
    )

    class Meta:
        abstract = True


class AuditableMixin(models.Model):

    created = models.DateTimeField(
        default=timezone.now,
        editable=False,
    )

    modified = models.DateTimeField(
        default=timezone.now,
        editable=False,
    )

    class Meta:
        abstract = True


class ActivatedMixin(models.Model):
    """
    """
    is_active = models.BooleanField(default=False)

    class Meta:
        abstract = True


class ActivatedQuerySet(models.QuerySet):

    def not_activated(self):
        return self.filter(is_active=False)

    def activated(self):
        return self.filter(is_active=True)



from django.conf import settings

PAGE_CONFIG = {
    'products': [],
    'custom_page_admin': False,
    'autoslug': True,
    'landing': [
        {
            'type': (1, 'General'),
            'template': ''
        }
    ]
}

SLIDER_SIZE_PC = (1000, 500)
SLIDER_SIZE_TABLET = (1000, 500)
SLIDER_SIZE_MOBILE = (1000, 500)
SLIDER_SIZE_SHARE = (1000, 500)

SLIDER_FORMAT = 'JPEG'

SLIDER_OPTIONS = {
    'quality': 90,
    'optimize': True,
    'progressive': True
}

SITE_URL = 'https://www.example.com'

PAGE_SITE_URL = getattr(settings, 'SITE_URL', SITE_URL)

PAGE_SLIDER_PC_SIZES = getattr(settings, 'SLIDER_SIZE_PC', SLIDER_SIZE_PC)
PAGE_SLIDER_TABLET_SIZES = getattr(settings, 'SLIDER_SIZE_TABLET', SLIDER_SIZE_TABLET)
PAGE_SLIDER_MOBILE_SIZES = getattr(settings, 'SLIDER_SIZE_MOBILE', SLIDER_SIZE_MOBILE)
PAGE_SLIDER_SHARE_SIZES = getattr(settings, 'SLIDER_SIZE_SHARE', SLIDER_SIZE_SHARE)

PAGE_SLIDER_FORMAT = getattr(settings, 'SLIDER_FORMAT', SLIDER_FORMAT)

PAGE_SLIDER_OPTIONS = getattr(settings, 'SLIDER_OPTIONS', SLIDER_OPTIONS)

SLIDER_TYPE_CONTENT = [
    'TEXT',
    'BUTTON',
]


PAGE_CONFIG = getattr(settings, 'PAGE_CONFIG', PAGE_CONFIG)

PAGE_TYPES = [config.get('type') for config in PAGE_CONFIG.get('landing')]

PAGE_PRODUCTS = PAGE_CONFIG.get('products')

PAGE_CUSTOM_ADMIN = PAGE_CONFIG.get('custom_page_admin')

PAGE_AUTOSLUG = PAGE_CONFIG.get('autoslug')


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0013_auto_20170630_1104'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='description',
            field=models.CharField(max_length=100, null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='subtitle_custom_color',
            field=models.CharField(max_length=7, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='title',
            field=models.CharField(max_length=200, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='title_custom_color',
            field=models.CharField(max_length=7, null=True, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from pages.models.page import Page
from pages.models.page import PageTimer
from pages.api.page.schema.nodes.timer import TimerNode
from pages.api.slider.schema.nodes.slider import SliderNode


class PageNode(DjangoObjectType):

    sliders = graphene.List(SliderNode)
    timer = graphene.Field(TimerNode)

    class Meta:
        model = Page

    def resolve_sliders(self, args, request, info):
        return self.slidercontent_set.all()

    def resolve_timer(self, args, request, info):
        page_timer = None
        page_timers = PageTimer.objects.filter(identifier=self.timer_id)
        if page_timers.exists():
            page_timer = page_timers.first()
        return page_timer



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0017_auto_20171219_1524'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='product_categories',
            field=models.CharField(max_length=300, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='page',
            name='video_provider',
            field=models.CharField(blank=True, max_length=200, null=True, choices=[(b'Wistia', b'Wistia'), (b'Vimeo', b'Vimeo')]),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import pages.utils.image
import imagekit.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0002_insert_default_data'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='picture_mobile',
            field=imagekit.models.fields.ProcessedImageField(null=True, upload_to=pages.utils.image.upload_picture_to),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='page',
            name='picture_pc',
            field=imagekit.models.fields.ProcessedImageField(default=1, upload_to=pages.utils.image.upload_picture_to),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='page',
            name='picture_share',
            field=imagekit.models.fields.ProcessedImageField(null=True, upload_to=pages.utils.image.upload_picture_to),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='page',
            name='picture_tablet',
            field=imagekit.models.fields.ProcessedImageField(null=True, upload_to=pages.utils.image.upload_picture_to),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='page',
            name='type',
            field=models.SmallIntegerField(default=1, choices=[(1, b'General')]),
            preserve_default=True,
        ),
    ]



import os
import uuid

from imagekit.processors import ResizeToFill

from django.utils import timezone


def get_slider_sizes(sizes):
    slider_sizes = []
    slider_sizes.append(ResizeToFill(width=sizes[0], height=sizes[1]))
    return slider_sizes


def upload_picture_to(obj, filename):
    f, ext = os.path.splitext(filename)
    _uuid = str(uuid.uuid4()).replace('-', '')
    url = 'pages/slider/%(date_now)s/%(uuid)s%(ext)s' % {
        'date_now': timezone.now().strftime('%Y/%m/%d'),
        'uuid': _uuid,
        'ext': ext
    }
    return url


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def insert_default_data(apps, schema_editor):
    SliderContentType = apps.get_model('pages', 'SliderContentType')
    data = [
        "Text",
        "Description",
        "Button"
    ]

    for slider_type in data:
        slider_content_type = SliderContentType()
        slider_content_type.name = slider_type
        slider_content_type.save()


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0001_initial'),
    ]
    operations = [
        migrations.RunPython(insert_default_data)
    ]
